Package: polite
Version: 0.1.4
Title: Be Nice on the Web
Description: Be responsible when scraping data from websites by following polite principles: introduce yourself, ask for permission, take slowly and never ask twice.
Authors@R: person(given="Dmytro", family="Perepolkin", email = "dperepolkin@gmail.com", role = c("aut", "cre"), comment = c(ORCID="0000-0001-8558-6183"))
License: MIT + file LICENSE
Encoding: UTF-8
ByteCompile: true
URL: https://github.com/dmi3kno/polite,
        https://dmi3kno.github.io/polite/
BugReports: https://github.com/dmi3kno/polite/issues
Imports: httr, magrittr, memoise, ratelimitr, robotstxt, rvest, stats,
        usethis
Suggests: dplyr, testthat, covr, webmockr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-10 19:50:22 UTC; DmytroPerepolkin
Author: Dmytro Perepolkin [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8558-6183>)
Maintainer: Dmytro Perepolkin <dperepolkin@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-11 05:10:02 UTC
Built: R 4.5.2; ; 2026-08-22 14:48:57 UTC; unix
